@@ -90,6 +90,9 @@ template<typename F>
 struct is_function_pointer : public ::std::integral_constant<bool
 	, ::std::is_pointer<F>::value && ::std::is_function< typename ::std::remove_pointer<F>::type >::value > {};
 
+template<bool B>
+struct bool_constant : ::std::integral_constant<bool, B> {};
+
 #else
 
 //======================================================================
@@ -357,7 +360,7 @@ public:
 }
 
 /**
-* @brief	is class
+ * @brief	is class
 */
 template<typename T>
 struct is_class : public is_class_helper::is_class<T>::type
@@ -458,7 +461,7 @@ public:
 }
 
 /**
-* @brief	is base of
+ * @brief	is base of
 */
 template<typename Base, typename Derived>
 struct is_base_of : public is_base_of_helper::is_base_of<Base, Derived>::type
@@ -660,7 +663,7 @@ public:
 }
 
 /**
-* @brief	is member function pointer
+ * @brief	is member function pointer
 */
 template<typename T>
 struct is_member_function_pointer : public is_member_function_pointer_helper::is_member_function_pointer<T>::type
@@ -697,6 +700,61 @@ struct is_member_pointer : public is_member_pointer_helper::is_member_pointer<T>
 
 #endif // #if IUTEST_HAS_HDR_TYPETRAITS
 
+#if !defined(IUTEST_NO_ARGUMENT_DEPENDENT_LOOKUP)
+
+namespace has_equal_to_operator_helper
+{
+	typedef char no_t[7];
+	template<typename T1, typename T2>
+	no_t& operator == (const T1& lhs, const T2& rhs);
+}
+
+namespace has_equal_to_operator_impl
+{
+	using namespace has_equal_to_operator_helper;
+	/** @private */
+	template<typename T>
+	struct has_equal_to_operator
+	{
+		typedef bool_constant< (sizeof(*(T*)0 == *(T*)0) != sizeof(has_equal_to_operator_helper::no_t) ) > type;
+	};
+}
+
+/**
+ * @brief	has equal to operator
+*/
+template<typename T>
+struct has_equal_to : public has_equal_to_operator_impl::has_equal_to_operator<T>::type
+{
+};
+
+namespace has_not_equal_to_operator_helper
+{
+	typedef char no_t[7];
+	template<typename T1, typename T2>
+	no_t& operator != (const T1& lhs, const T2& rhs);
+}
+
+namespace has_not_equal_to_operator_impl
+{
+	using namespace has_not_equal_to_operator_helper;
+	/** @private */
+	template<typename T>
+	struct has_not_equal_to_operator
+	{
+		typedef bool_constant< (sizeof(*(T*)0 != *(T*)0) != sizeof(has_not_equal_to_operator_helper::no_t)) > type;
+	};
+}
+
+/**
+ * @brief	has not equal to operator
+*/
+template<typename T>
+struct has_not_equal_to : public has_not_equal_to_operator_impl::has_not_equal_to_operator<T>::type
+{
+};
+
+#endif
 
 #if !defined(IUTEST_NO_TEMPLATE_PARTIAL_SPECIALIZATION)
 
@@ -733,7 +791,7 @@ class function_return_type
 	IIUT_DECL_FUNCTION_RETURN_TYPE_I(n, IUTEST_PP_EMPTY());	\
 	IIUT_DECL_FUNCTION_RETURN_TYPE_I(n, const);	\
 	IIUT_DECL_FUNCTION_RETURN_TYPE_I(n, volatile);	\
-	IIUT_DECL_FUNCTION_RETURN_TYPE_I(n, const volatile)	\
+	IIUT_DECL_FUNCTION_RETURN_TYPE_I(n, const volatile)
 
 #define IIUT_DECL_FUNCTION_RETURN_TYPE_I(n, CV)	\
 	template<typename R, typename U, IUTEST_PP_ENUM_PARAMS(n, typename T)>struct impl<R (U::*)(IUTEST_PP_ENUM_PARAMS(n, T)) CV>	\
@@ -787,5 +845,4 @@ public:
 
 }	// end of namespace iutest_type_traits
 
-
 #endif // INCG_IRIS_IUTEST_TYPE_TRAITS_HPP_6F091F15_784A_4F50_BD18_B8F67C5AF0CF_
