@@ -43,6 +43,12 @@ public:
 		TestEnv::event_listeners().set_default_xml_generator(NULL);
 	}
 public:
+	/**
+	 * @brief	出力ファイルパスの取得
+	*/
+	const ::std::string& GetFilePath(void) const { return m_output_path; }
+
+private:
 	/**
 	 * @brief	出力ファイルの設定
 	*/
@@ -55,22 +61,23 @@ public:
 		else
 		{
 			m_output_path = path;
-			if( path.find('.') == ::std::string::npos )
+			const ::std::string::size_type pos = path.find('.');
+			if( pos == ::std::string::npos
+				|| pos == path.length()-1 )
 			{
-				m_output_path += "\\";
+				m_output_path += detail::GetPathSeparator();
 				m_output_path += detail::kStrings::DefaultXmlReportFileName;
 			}
 		}
 	}
-	/**
-	 * @brief	出力ファイルパスの取得
-	*/
-	const ::std::string& GetFilePath(void) const { return m_output_path; }
 
 public:
 	virtual void OnTestProgramStart(const UnitTest& test) IUTEST_CXX_OVERRIDE;
 	virtual void OnTestProgramEnd(const UnitTest& test) IUTEST_CXX_OVERRIDE;
 
+private:
+	virtual void OnReportTest(IFile* file, const UnitTest& test);
+
 private:
 	static void OnReportTestCase(IFile* file, const TestCase& test_case);
 	static void OnReportTestInfo(IFile* file, const TestInfo& test_info);
@@ -82,15 +89,19 @@ private:
 	bool FileOpen(const char* path);
 	void FileClose(void);
 
-private:
+protected:
 	static void OutputXmlCDataSection(IFile* file, const char* data);
 	static void OutputXmlAttribute(IFile* file, const char* name, const char* value);
-
-private:
+	
+protected:
 	static ::std::string EscapeXmlAttribute(const char* str)
 	{
 		return EscapeXml(str, true);
 	}
+	static ::std::string EscapeXmlAttribute(const ::std::string str)
+	{
+		return EscapeXml(str.c_str(), true);
+	}
 	static ::std::string EscapeXmlText(const char* str)
 	{
 		return EscapeXml(str, false);
@@ -126,13 +137,7 @@ public:
 		::std::string xmlpath = TestEnv::get_report_xml_filepath();
 		if(!xmlpath.empty())
 		{
-			DefaultXmlGeneratorListener* listener = reinterpret_cast<DefaultXmlGeneratorListener*>(TestEnv::event_listeners().default_xml_generator());
-			if(listener == NULL)
-			{
-				TestEnv::event_listeners().set_default_xml_generator(new DefaultXmlGeneratorListener(xmlpath));
-				return;
-			}
-			listener->SetFilePath(xmlpath);
+			TestEnv::event_listeners().set_default_xml_generator(new DefaultXmlGeneratorListener(xmlpath));
 		}
 	}
 };
