@@ -4,9 +4,7 @@
  * @file		iutest_util_assertion.hpp
  * @brief		iris unit test assertion 拡張 ファイル
  *
- * @author		t.sirayanagi
- * @version		1.0
- *
+ * @author		t.shirayanagi
  * @par			copyright
  * Copyright (C) 2012-2014, Takazumi Shirayanagi\n
  * This software is released under the new BSD License,
@@ -21,9 +19,6 @@
 // include
 #include "../internal/iutest_type_traits.hpp"
 
-namespace iuutil
-{
-
 //======================================================================
 // define
 /**
@@ -146,37 +141,37 @@ namespace iuutil
 #endif
 
 /**
-* @ingroup	IUTEST_ASSERT_
-* @brief	文字列部分一致 テスト
-* @param	substr	= 部分文字列
-* @param	actual	= 検査対象文字列
+ * @ingroup	IUTEST_ASSERT_
+ * @brief	文字列部分一致 テスト
+ * @param	substr	= 部分文字列
+ * @param	actual	= 検査対象文字列
 */
 #ifndef IUTEST_ASSERT_STRIN
 #  define	IUTEST_ASSERT_STRIN(substr, actual)		IUTEST_TEST_STRIN(substr, actual, IUTEST_ASSERT_FAILURE)
 #endif
 /**
-* @ingroup	IUTEST_EXPECT_
-* @brief	文字列部分一致 テスト
-* @param	substr	= 部分文字列
-* @param	actual	= 検査対象文字列
+ * @ingroup	IUTEST_EXPECT_
+ * @brief	文字列部分一致 テスト
+ * @param	substr	= 部分文字列
+ * @param	actual	= 検査対象文字列
 */
 #ifndef IUTEST_EXPECT_STRIN
 #  define	IUTEST_EXPECT_STRIN(substr, actual)		IUTEST_TEST_STRIN(substr, actual, IUTEST_EXPECT_FAILURE)
 #endif
 /**
-* @ingroup	IUTEST_INFORM
-* @brief	文字列部分一致 テスト
-* @param	substr	= 部分文字列
-* @param	actual	= 検査対象文字列
+ * @ingroup	IUTEST_INFORM
+ * @brief	文字列部分一致 テスト
+ * @param	substr	= 部分文字列
+ * @param	actual	= 検査対象文字列
 */
 #ifndef IUTEST_INFORM_STRIN
 #  define	IUTEST_INFORM_STRIN(substr, actual)		IUTEST_TEST_STRIN(substr, actual, IUTEST_INFORM_FAILURE)
 #endif
 /**
-* @ingroup	IUTEST_ASSUME
-* @brief	文字列部分一致 テスト
-* @param	substr	= 部分文字列
-* @param	actual	= 検査対象文字列
+ * @ingroup	IUTEST_ASSUME
+ * @brief	文字列部分一致 テスト
+ * @param	substr	= 部分文字列
+ * @param	actual	= 検査対象文字列
 */
 #ifndef IUTEST_ASSUME_STRIN
 #  define	IUTEST_ASSUME_STRIN(substr, actual)		IUTEST_TEST_STRIN(substr, actual, IUTEST_ASSUME_FAILURE)
@@ -184,42 +179,48 @@ namespace iuutil
 
 
 /**
-* @ingroup	IUTEST_ASSERT_
-* @brief	文字列部分一致 テスト
-* @param	substr	= 部分文字列
-* @param	actual	= 検査対象文字列
+ * @ingroup	IUTEST_ASSERT_
+ * @brief	文字列部分一致 テスト
+ * @param	substr	= 部分文字列
+ * @param	actual	= 検査対象文字列
 */
 #ifndef IUTEST_ASSERT_STRNOTIN
 #  define	IUTEST_ASSERT_STRNOTIN(substr, actual)		IUTEST_TEST_STRNOTIN(substr, actual, IUTEST_ASSERT_FAILURE)
 #endif
 /**
-* @ingroup	IUTEST_EXPECT_
-* @brief	文字列部分一致 テスト
-* @param	substr	= 部分文字列
-* @param	actual	= 検査対象文字列
+ * @ingroup	IUTEST_EXPECT_
+ * @brief	文字列部分一致 テスト
+ * @param	substr	= 部分文字列
+ * @param	actual	= 検査対象文字列
 */
 #ifndef IUTEST_EXPECT_STRNOTIN
 #  define	IUTEST_EXPECT_STRNOTIN(substr, actual)		IUTEST_TEST_STRNOTIN(substr, actual, IUTEST_EXPECT_FAILURE)
 #endif
 /**
-* @ingroup	IUTEST_INFORM
-* @brief	文字列部分一致 テスト
-* @param	substr	= 部分文字列
-* @param	actual	= 検査対象文字列
+ * @ingroup	IUTEST_INFORM
+ * @brief	文字列部分一致 テスト
+ * @param	substr	= 部分文字列
+ * @param	actual	= 検査対象文字列
 */
 #ifndef IUTEST_INFORM_STRNOTIN
 #  define	IUTEST_INFORM_STRNOTIN(substr, actual)		IUTEST_TEST_STRNOTIN(substr, actual, IUTEST_INFORM_FAILURE)
 #endif
 /**
-* @ingroup	IUTEST_ASSUME
-* @brief	文字列部分一致 テスト
-* @param	substr	= 部分文字列
-* @param	actual	= 検査対象文字列
+ * @ingroup	IUTEST_ASSUME
+ * @brief	文字列部分一致 テスト
+ * @param	substr	= 部分文字列
+ * @param	actual	= 検査対象文字列
 */
 #ifndef IUTEST_ASSUME_STRNOTIN
 #  define	IUTEST_ASSUME_STRNOTIN(substr, actual)		IUTEST_TEST_STRNOTIN(substr, actual, IUTEST_ASSUME_FAILURE)
 #endif
 
+/**
+ * @private
+ * @brief	for gtest
+ * @{
+*/
+
 #define ASSERT_EQ_COLLECTIONS	IUTEST_ASSERT_EQ_COLLECTIONS
 #define ASSERT_EQ_RANGE			IUTEST_ASSERT_EQ_RANGE
 #define ASSERT_STRLNEQ			IUTEST_ASSERT_STRLNEQ
@@ -244,6 +245,10 @@ namespace iuutil
 #define ASSUME_STRIN			IUTEST_ASSUME_STRIN
 #define ASSUME_STRNOTIN			IUTEST_ASSUME_STRNOTIN
 
+/**
+ * @}
+*/
+
 /**
  * @private
  * @{
@@ -259,6 +264,9 @@ namespace iuutil
  * @}
 */
 
+namespace iuutil
+{
+
 //======================================================================
 // function
 /**
@@ -308,7 +316,7 @@ template<typename T1, typename T2>
 */
 template<typename T1, typename T2>
 ::iutest::AssertionResult IUTEST_ATTRIBUTE_UNUSED_ CmpHelperEqCollections(const char* expr1b, const char* expr1e, const char* expr2b, const char* expr2e
-							   , T1 b1, T1 e1, T2 b2, T2 e2)
+							, T1 b1, T1 e1, T2 b2, T2 e2)
 {
 	if( ::iutest::AssertionResult ar = CmpHelperEqIterator(b1, e1, b2, e2) )
 		;
@@ -336,18 +344,16 @@ template<typename T1, typename T2>
 }
 
 /**
-* @brief	Equal Range Helper
+ * @brief	Equal Range Helper
 */
 template<typename T1, typename T2>
 ::iutest::AssertionResult IUTEST_ATTRIBUTE_UNUSED_ CmpHelperEqRange(const char* expected_expr, const char* actual_expr
 	, T1 expected, T2 actual)
 {
-//	return detail::CmpHelperEqRange(expected_expr, actual_expr, ::std::begin(expected), ::std::end(expected)
-//		, ::std::begin(actual), ::std::end(actual));
 	return detail::CmpHelperEqRange(expected_expr, actual_expr, expected.begin(), expected.end()
 		, actual.begin(), actual.end());
 }
-#if !defined(IUTEST_NO_TEMPLATE_PARTIAL_SPECIALIZATION)
+#if !defined(IUTEST_NO_FUNCTION_TEMPLATE_ORDERING)
 
 /** @overload */
 template<typename T1, size_t SIZE1, typename T2, size_t SIZE2>
@@ -380,7 +386,7 @@ template<typename T1, size_t SIZE1, typename T2>
  * @brief	文字列長アサーションフォーマッター
 */
 inline ::iutest::AssertionResult IUTEST_ATTRIBUTE_UNUSED_ CmpHelperSTRLNEQ(const char* expr1, const char* expr2
-										 , size_t len1, const char* val2)
+										, size_t len1, const char* val2)
 {
 	const size_t len2 = strlen(val2);
 	if( len2 == len1 )
@@ -394,7 +400,7 @@ inline ::iutest::AssertionResult IUTEST_ATTRIBUTE_UNUSED_ CmpHelperSTRLNEQ(const
  * @brief	文字列長アサーションフォーマッター
 */
 inline ::iutest::AssertionResult IUTEST_ATTRIBUTE_UNUSED_ CmpHelperSTRLNEQ(const char* expr1, const char* expr2
-										 , size_t len1, const wchar_t* val2)
+										, size_t len1, const wchar_t* val2)
 {
 	const size_t len2 = wcslen(val2);
 	if( len2 == len1 )
@@ -405,88 +411,138 @@ inline ::iutest::AssertionResult IUTEST_ATTRIBUTE_UNUSED_ CmpHelperSTRLNEQ(const
 		<< "\n  Actual: " << val2 << " : " << len2 << "\nExpected: " << len1 ;
 }
 
+namespace StrInHelper
+{
+
+inline bool IUTEST_ATTRIBUTE_UNUSED_ Compare(const char* substr, const char* actual)
+{
+	if( substr == NULL || actual == NULL )
+	{
+		return substr == actual;
+	}
+	return strstr(actual, substr) != NULL;
+}
+
+inline bool IUTEST_ATTRIBUTE_UNUSED_ Compare(const wchar_t* substr, const wchar_t* actual)
+{
+	if( substr == NULL || actual == NULL )
+	{
+		return substr == actual;
+	}
+	return wcsstr(actual, substr) != NULL;
+}
+
+template<typename Elem, typename Traits, typename Ax>
+inline bool IUTEST_ATTRIBUTE_UNUSED_ Compare(const ::std::basic_string<Elem, Traits, Ax>& substr
+	, const ::std::basic_string<Elem, Traits, Ax>& actual)
+{
+	return Compare(substr.c_str(), actual.c_str());
+}
+template<typename Elem, typename Traits, typename Ax>
+inline bool IUTEST_ATTRIBUTE_UNUSED_ Compare(const Elem* substr
+	, const ::std::basic_string<Elem, Traits, Ax>& actual)
+{
+	return Compare(substr, actual.c_str());
+}
+template<typename Elem, typename Traits, typename Ax>
+inline bool IUTEST_ATTRIBUTE_UNUSED_ Compare(const ::std::basic_string<Elem, Traits, Ax>& substr
+	, const Elem* actual)
+{
+	return Compare(substr.c_str(), actual);
+}
+
+template<typename T1, typename T2>
+inline ::iutest::AssertionResult IUTEST_ATTRIBUTE_UNUSED_ Assertion(const char* substr_str, const char* actual_str
+	, const T1& substr, const T2& actual)
+{
+	if( Compare(substr, actual) )
+	{
+		return ::iutest::AssertionSuccess();
+	}
+
+	return ::iutest::AssertionFailure() << "error: Expected: " << "strstr(" << actual_str << ", " << substr_str << ") != NULL"
+		<< "\n  Actual: " << "strstr(\"" << actual << "\", " << substr << ") == NULL";
+}
+
+}
+
 /**
-* @brief	文字列部分一致アサーションフォーマッター
+ * @brief	文字列部分一致アサーションフォーマッター
 */
 inline ::iutest::AssertionResult IUTEST_ATTRIBUTE_UNUSED_ CmpHelperSTRIN(const char* substr_str, const char* actual_str
 	, const char* substr, const char* actual)
 {
-	if( substr == NULL || actual == NULL )
-	{
-		if( substr == actual )
-		{
-			return ::iutest::AssertionSuccess();
-		}
-	}
-	else if( strstr(actual, substr) != NULL )
-	{
-		return ::iutest::AssertionSuccess();
-	}
-	return ::iutest::AssertionFailure() << "error: Expected: " << "strstr(" << actual_str << ", " << substr_str << ") != NULL"
-		<< "\n  Actual: " << "strstr(" << actual << ", " << substr << ") == NULL";
+	return StrInHelper::Assertion(substr_str, actual_str, substr, actual);
 }
 
-/**
-* @brief	文字列部分一致アサーションフォーマッター
-*/
 inline ::iutest::AssertionResult IUTEST_ATTRIBUTE_UNUSED_ CmpHelperSTRIN(const char* substr_str, const char* actual_str
 	, const wchar_t* substr, const wchar_t* actual)
 {
-	if( substr == NULL || actual == NULL )
-	{
-		if( substr == actual )
-		{
-			return ::iutest::AssertionSuccess();
-		}
-	}
-	else if( wcsstr(actual, substr) != NULL )
+	return StrInHelper::Assertion(substr_str, actual_str, substr, actual);
+}
+
+template<typename Elem, typename Traits, typename Ax>
+inline ::iutest::AssertionResult IUTEST_ATTRIBUTE_UNUSED_ CmpHelperSTRIN(const char* substr_str, const char* actual_str
+																, const ::std::basic_string<Elem, Traits, Ax>& substr
+																, const ::std::basic_string<Elem, Traits, Ax>& actual)
+{
+	return StrInHelper::Assertion(substr_str, actual_str, substr, actual);
+}
+template<typename Elem, typename Traits, typename Ax>
+inline ::iutest::AssertionResult IUTEST_ATTRIBUTE_UNUSED_ CmpHelperSTRIN(const char* substr_str, const char* actual_str
+																, const Elem* substr
+																, const ::std::basic_string<Elem, Traits, Ax>& actual)
+{
+	return StrInHelper::Assertion(substr_str, actual_str, substr, actual);
+}
+template<typename Elem, typename Traits, typename Ax>
+inline ::iutest::AssertionResult IUTEST_ATTRIBUTE_UNUSED_ CmpHelperSTRIN(const char* substr_str, const char* actual_str
+																, const ::std::basic_string<Elem, Traits, Ax>& substr
+																, const Elem* actual)
+{
+	return StrInHelper::Assertion(substr_str, actual_str, substr, actual);
+}
+
+namespace StrNotInHelper
+{
+
+template<typename T1, typename T2>
+inline bool IUTEST_ATTRIBUTE_UNUSED_ Compare(const T1& substr, const T2& actual)
+{
+	return !StrInHelper::Compare(substr, actual);
+}
+
+template<typename T1, typename T2>
+inline ::iutest::AssertionResult IUTEST_ATTRIBUTE_UNUSED_ Assertion(const char* substr_str, const char* actual_str
+	, const T1& substr, const T2& actual)
+{
+	if( Compare(substr, actual) )
 	{
 		return ::iutest::AssertionSuccess();
 	}
-	return ::iutest::AssertionFailure() << "error: Expected: " << "strstr(" << actual_str << ", " << substr_str << ") != NULL"
-		<< "\n  Actual: " << "strstr(" << actual << ", " << substr << ") == NULL";
+
+	return ::iutest::AssertionFailure() << "error: Expected: " << "strstr(" << actual_str << ", " << substr_str << ") == NULL"
+		<< "\n  Actual: " << "strstr(\"" << actual << "\", " << substr << ") != NULL";
+}
+
 }
 
 /**
-* @brief	文字列部分一致アサーションフォーマッター
+ * @brief	文字列部分一致アサーションフォーマッター
 */
 inline ::iutest::AssertionResult IUTEST_ATTRIBUTE_UNUSED_ CmpHelperSTRNOTIN(const char* substr_str, const char* actual_str
 	, const char* substr, const char* actual)
 {
-	if( substr == NULL || actual == NULL )
-	{
-		if( substr != actual )
-		{
-			return ::iutest::AssertionSuccess();
-		}
-	}
-	else if( strstr(actual, substr) == NULL )
-	{
-		return ::iutest::AssertionSuccess();
-	}
-	return ::iutest::AssertionFailure() << "error: Expected: " << "strstr(" << actual_str << ", " << substr_str << ") == NULL"
-		<< "\n  Actual: " << "strstr(" << actual << ", " << substr << ") != NULL";
+	return StrNotInHelper::Assertion(substr_str, actual_str, substr, actual);
 }
 
 /**
-* @brief	文字列部分一致アサーションフォーマッター
+ * @brief	文字列部分一致アサーションフォーマッター
 */
 inline ::iutest::AssertionResult IUTEST_ATTRIBUTE_UNUSED_ CmpHelperSTRNOTIN(const char* substr_str, const char* actual_str
 	, const wchar_t* substr, const wchar_t* actual)
 {
-	if( substr == NULL || actual == NULL )
-	{
-		if( substr != actual )
-		{
-			return ::iutest::AssertionSuccess();
-		}
-	}
-	else if( wcsstr(actual, substr) == NULL )
-	{
-		return ::iutest::AssertionSuccess();
-	}
-	return ::iutest::AssertionFailure() << "error: Expected: " << "strstr(" << actual_str << ", " << substr_str << ") == NULL"
-		<< "\n  Actual: " << "strstr(" << actual << ", " << substr << ") != NULL";
+	return StrNotInHelper::Assertion(substr_str, actual_str, substr, actual);
 }
 
 /**
@@ -495,7 +551,7 @@ inline ::iutest::AssertionResult IUTEST_ATTRIBUTE_UNUSED_ CmpHelperSTRNOTIN(cons
 */
 #define IIUT_COMPARE_HELPER_DEC_(name)	struct name {										\
 	template<typename T1, typename T2>														\
-	static ::iutest::AssertionResult Comp(const char* expr1, const char* expr2, const T1& val1, const T2& val2)	{	\
+	static ::iutest::AssertionResult Comp(const char* expr1, const char* expr2, const T1& val1, const T2& val2) {	\
 		return ::iutest::internal::CmpHelper##name(expr1, expr2, val1, val2);				\
 	}																						\
 	}
@@ -503,89 +559,6 @@ inline ::iutest::AssertionResult IUTEST_ATTRIBUTE_UNUSED_ CmpHelperSTRNOTIN(cons
  * @}
 */
 
-/**
- * @brief	OR テスト用オブジェクト郡
-*/
-struct CmpHelper
-{
-	IIUT_COMPARE_HELPER_DEC_(EQ);	//!< OR テスト用 EQ
-	IIUT_COMPARE_HELPER_DEC_(NE);	//!< OR テスト用 NE
-	IIUT_COMPARE_HELPER_DEC_(LE);	//!< OR テスト用 LE
-	IIUT_COMPARE_HELPER_DEC_(LT);	//!< OR テスト用 LT
-	IIUT_COMPARE_HELPER_DEC_(GE);	//!< OR テスト用 GE
-	IIUT_COMPARE_HELPER_DEC_(GT);	//!< OR テスト用 GT
-};
-
-#undef IIUT_COMPARE_HELPER_DEC_
-
-/**
- * @brief	OR テスト用比較ヘルパー
-*/
-template<typename COMP>
-struct CmpHelperOR
-{
-private:
-	template<typename T, typename DMY>
-	struct CompImpl
-	{
-		template<typename T1, typename T2>
-		static ::iutest::AssertionResult Comp(const char* expr1, const char* expr2
-			, T1 val1, T2 val2)
-		{
-			return T::Comp(expr1, expr2, val1, val2);
-		}
-	};
-	template<typename DMY>
-	struct CompImpl<CmpHelper::EQ, DMY>
-	{
-		template<typename T1, typename T2>
-		static ::iutest::AssertionResult Comp(const char* expr1, const char* expr2
-			, T1 val1, T2 val2)
-		{
-			// val1 == actual
-			return CmpHelper::EQ::Comp(expr2, expr1, val2, val1);
-		}
-	};
-public:
-
-	/**
-	 * @brief	フォーマッター
-	*/
-	template<typename T1, typename T2>
-	static ::iutest::AssertionResult Comp(const char* expr1, const char* expr2
-		, T1 val1, T2 val2)
-	{
-		return CompImpl<COMP, void>::Comp(expr1, expr2, val1, val2);
-	}
-	/** @overload */
-	template<typename T1, typename T2, typename T3>
-	static ::iutest::AssertionResult Comp(const char* expr1, const char* expr2, const char* expr3
-		, T1 val1, T2 val2, T3 val3)
-	{
-		typename ::iutest_type_traits::add_rvalue_reference< ::iutest::AssertionResult >::type ar = Comp(expr1, expr2, val1, val2);
-		if( ar ) return ar;
-		return Comp(expr1, expr3, val1, val3) << "\n" << ar.message();
-	}
-	/** @overload */
-	template<typename T1, typename T2, typename T3, typename T4>
-	static ::iutest::AssertionResult Comp(const char* expr1, const char* expr2, const char* expr3, const char* expr4
-		, T1 val1, T2 val2, T3 val3, T4 val4)
-	{
-		typename ::iutest_type_traits::add_rvalue_reference< ::iutest::AssertionResult >::type ar =  Comp(expr1, expr2, expr3, val1, val2, val3);
-		if( ar ) return ar;
-		return Comp(expr1, expr4, val1, val4) << "\n" << ar.message();
-	}
-	/** @overload */
-	template<typename T1, typename T2, typename T3, typename T4, typename T5>
-	static ::iutest::AssertionResult Comp(const char* expr1, const char* expr2, const char* expr3, const char* expr4, const char* expr5
-		, T1 val1, T2 val2, T3 val3, T4 val4, T5 val5)
-	{
-		typename ::iutest_type_traits::add_rvalue_reference< ::iutest::AssertionResult >::type ar =  Comp(expr1, expr2, expr3, expr4, val1, val2, val3, val4);
-		if( ar ) return ar;
-		return Comp(expr1, expr5, val1, val5) << "\n" << ar.message();
-	}
-};
-
 }	// end of namespace iuutil
 
-#endif	// INCG_IRIS_IUTEST_UTIL_ASSERTION_HPP_A45F8265_40E0_44F2_91C6_090B1778657C_
+#endif // INCG_IRIS_IUTEST_UTIL_ASSERTION_HPP_A45F8265_40E0_44F2_91C6_090B1778657C_
