@@ -15,7 +15,7 @@
 
 //======================================================================
 // include
-#include "../include/iutest.hpp"
+#include "iutest.hpp"
 
 template<int A, int B>
 struct add_t { enum result { value = A + B }; };
@@ -66,9 +66,10 @@ struct Point
 IUTEST(AssertionTest, EQ)
 {
 	int x0=0, y0=0;
+	char z0=0;
 	IUTEST_ASSERT_EQ(x0, y0);
-	IUTEST_EXPECT_EQ(x0, y0);
-	IUTEST_INFORM_EQ(x0, y0);
+	IUTEST_EXPECT_EQ(x0, z0);
+	IUTEST_INFORM_EQ(z0, y0);
 	int* zero=NULL;
 	IUTEST_ASSERT_EQ(NULL, zero);
 #if IUTEST_HAS_NULLPTR
@@ -100,6 +101,29 @@ IUTEST(AssertionTest, EQ)
 #endif
 }
 
+#if IUTEST_HAS_ASSERTION_NOEQUALTO_OBJECT
+
+struct TestObjectX
+{
+	int a,b,c;
+};
+
+IUTEST(AssertionTest, MemCmpEQ)
+{
+	TestObjectX x={0,1,2};
+	TestObjectX y={0,1,2};
+	IUTEST_ASSERT_EQ(x, y);
+}
+
+IUTEST(AssertionTest, MemCmpNE)
+{
+	TestObjectX x={0,1,2};
+	TestObjectX y={0,1,3};
+	IUTEST_ASSERT_NE(x, y);
+}
+
+#endif
+
 IUTEST(AssertionTest, NE)
 {
 	int x0=0, x1=1;
@@ -225,104 +249,6 @@ IUTEST(AssertionTest, HResultFailed)
 
 #endif
 
-#if IUTEST_HAS_EXCEPTIONS
-
-namespace assertion_test
-{
-
-static void	ExceptionFunction(int i)
-{
-	switch( i )
-	{
-	case 0:
-		return;
-	case 1:
-		throw 2;
-		break;
-	case 2:
-		throw ::std::bad_exception();
-		break;
-	case 3:
-		throw "error";
-	case 4:
-		throw ::std::string("error");
-	case 5:
-		throw 0.1f;
-	default:
-		break;
-	}
-}
-
-IUTEST(AssertionTest, ExceptionThrow)
-{
-	//IUTEST_ASSERT_THROW(throw ::std::bad_exception(), ::std::bad_exception);
-	IUTEST_ASSERT_THROW(ExceptionFunction(2), ::std::bad_exception);
-	IUTEST_EXPECT_THROW(ExceptionFunction(2), ::std::bad_exception);
-	IUTEST_INFORM_THROW(ExceptionFunction(2), ::std::bad_exception);
-}
-
-IUTEST(AssertionTest, ExceptionAnyThrow)
-{
-	IUTEST_ASSERT_ANY_THROW(throw ::std::bad_exception());
-	IUTEST_ASSERT_ANY_THROW(throw 1);
-	IUTEST_ASSERT_ANY_THROW(ExceptionFunction(1));
-	IUTEST_EXPECT_ANY_THROW(ExceptionFunction(1));
-	IUTEST_INFORM_ANY_THROW(ExceptionFunction(1));
-}
-
-IUTEST(AssertionTest, ExceptionNoThrow)
-{
-	IUTEST_ASSERT_NO_THROW((void)0);
-	IUTEST_ASSERT_NO_THROW(ExceptionFunction(0));
-	IUTEST_EXPECT_NO_THROW(ExceptionFunction(0));
-	IUTEST_INFORM_NO_THROW(ExceptionFunction(0));
-}
-
-IUTEST(AssertionTest, ExceptionValueEQ)
-{
-	IUTEST_ASSERT_THROW_VALUE_EQ(throw 2, int, 2);
-	IUTEST_ASSERT_THROW_VALUE_EQ(ExceptionFunction(1), int, 2);
-	IUTEST_EXPECT_THROW_VALUE_EQ(ExceptionFunction(1), int, 2);
-	IUTEST_INFORM_THROW_VALUE_EQ(ExceptionFunction(1), int, 2);
-}
-	
-IUTEST(AssertionTest, ExceptionValueNE)
-{
-	IUTEST_ASSERT_THROW_VALUE_NE(throw 2, int, 0);
-	IUTEST_ASSERT_THROW_VALUE_NE(ExceptionFunction(1), int, 0);
-	IUTEST_EXPECT_THROW_VALUE_NE(ExceptionFunction(1), int, 0);
-	IUTEST_INFORM_THROW_VALUE_NE(ExceptionFunction(1), int, 0);
-}
-
-IUTEST(AssertionTest, ExceptionValueSTREQ)
-{
-	IUTEST_ASSERT_THROW_VALUE_STREQ(throw "error", const char *, "error");
-	IUTEST_ASSERT_THROW_VALUE_STREQ(ExceptionFunction(3), const char *, "error");
-	IUTEST_EXPECT_THROW_VALUE_STREQ(ExceptionFunction(3), const char *, "error");
-	IUTEST_INFORM_THROW_VALUE_STREQ(ExceptionFunction(3), const char *, "error");
-	IUTEST_ASSERT_THROW_VALUE_STREQ(throw ::std::string("error"), ::std::string, "error");
-	IUTEST_ASSERT_THROW_VALUE_STREQ(ExceptionFunction(4), ::std::string, "error");
-	IUTEST_EXPECT_THROW_VALUE_STREQ(ExceptionFunction(4), ::std::string, "error");
-	IUTEST_INFORM_THROW_VALUE_STREQ(ExceptionFunction(4), ::std::string, "error");
-}
-
-IUTEST(AssertionTest, ExceptionValueSTRCASEEQ)
-{
-	IUTEST_ASSERT_THROW_VALUE_STRCASEEQ(throw "error", const char *, "Error");
-	IUTEST_ASSERT_THROW_VALUE_STRCASEEQ(ExceptionFunction(3), const char *, "Error");
-	IUTEST_EXPECT_THROW_VALUE_STRCASEEQ(ExceptionFunction(3), const char *, "Error");
-	IUTEST_INFORM_THROW_VALUE_STRCASEEQ(ExceptionFunction(3), const char *, "Error");
-}
-
-IUTEST(AssertionTest, ExceptionValueFormat)
-{
-	IUTEST_ASSERT_THROW_PRED_FORMAT2(::iutest::internal::CmpHelperFloatingPointEQ<float>, ExceptionFunction(5), float, 0.1f);
-}
-
-}
-
-#endif
-
 IUTEST(AssertionTest, ScopedTrace)
 {
 	{
