@@ -15,7 +15,7 @@
 
 //======================================================================
 // include
-#include "../include/iutest.hpp"
+#include "iutest.hpp"
 
 #if IUTEST_HAS_EXCEPTIONS
 #include <stdexcept>
